@@ -1,21 +1,21 @@
 
 /*
- * $Id: net_db.c,v 1.139.2.4 2000/02/09 23:29:59 wessels Exp $
+ * $Id: net_db.c,v 1.152.2.4 2001/01/12 00:51:50 wessels Exp $
  *
  * DEBUG: section 38    Network Measurement Database
  * AUTHOR: Duane Wessels
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  the Regents of the University of California.  Please see the
- *  COPYRIGHT file for full details.  Squid incorporates software
- *  developed and/or copyrighted by other sources.  Please see the
- *  CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 typedef struct {
     peer *p;
     StoreEntry *e;
+    store_client *sc;
     request_t *r;
     off_t seen;
     off_t used;
@@ -60,7 +61,7 @@ static void netdbPurgeLRU(void);
 static netdbEntry *netdbLookupHost(const char *key);
 static net_db_peer *netdbPeerByName(const netdbEntry * n, const char *);
 static net_db_peer *netdbPeerAdd(netdbEntry * n, peer * e);
-static char *netdbPeerName(const char *name);
+static const char *netdbPeerName(const char *name);
 static IPH netdbSendPing;
 static QS sortPeerByRtt;
 static QS sortByRtt;
@@ -80,9 +81,9 @@ static void
 netdbHashInsert(netdbEntry * n, struct in_addr addr)
 {
     xstrncpy(n->network, inet_ntoa(networkFromInaddr(addr)), 16);
-    n->key = n->network;
+    n->hash.key = n->network;
     assert(hash_lookup(addr_table, n->network) == NULL);
-    hash_join(addr_table, (hash_link *) n);
+    hash_join(addr_table, &n->hash);
 }
 
 static void
@@ -100,12 +101,12 @@ static void
 netdbHostInsert(netdbEntry * n, const char *hostname)
 {
     net_db_name *x = memAllocate(MEM_NET_DB_NAME);
-    x->name = xstrdup(hostname);
+    x->hash.key = xstrdup(hostname);
     x->next = n->hosts;
     n->hosts = x;
     x->net_db_entry = n;
     assert(hash_lookup(host_table, hostname) == NULL);
-    hash_join(host_table, (hash_link *) x);
+    hash_join(host_table, &x->hash);
     n->link_count++;
 }
 
@@ -125,7 +126,7 @@ netdbHostDelete(const net_db_name * x)
 	}
     }
     hash_remove_link(host_table, (hash_link *) x);
-    xfree(x->name);
+    xfree(x->hash.key);
     memFree((void *) x, MEM_NET_DB_NAME);
 }
 
@@ -367,14 +368,24 @@ static void
 netdbSaveState(void *foo)
 {
     LOCAL_ARRAY(char, path, SQUID_MAXPATHLEN);
-    FILE *fp;
+    Logfile *lf;
     netdbEntry *n;
     net_db_name *x;
     struct timeval start = current_time;
     int count = 0;
     snprintf(path, SQUID_MAXPATHLEN, "%s/netdb_state", storeSwapDir(0));
-    fp = fopen(path, "w");
-    if (fp == NULL) {
+    /*
+     * This was nicer when we were using stdio, but thanks to
+     * Solaris bugs, its a bad idea.  fopen can fail if more than
+     * 256 FDs are open.
+     */
+    /*
+     * unlink() is here because there is currently no way to make
+     * logfileOpen() use O_TRUNC.
+     */
+    unlink(path);
+    lf = logfileOpen(path, 4096, 0);
+    if (NULL == lf) {
 	debug(50, 1) ("netdbSaveState: %s: %s\n", path, xstrerror());
 	return;
     }
@@ -382,7 +393,7 @@ netdbSaveState(void *foo)
     while ((n = (netdbEntry *) hash_next(addr_table))) {
 	if (n->pings_recv == 0)
 	    continue;
-	fprintf(fp, "%s %d %d %10.5f %10.5f %d %d",
+	logfilePrintf(lf, "%s %d %d %10.5f %10.5f %d %d",
 	    n->network,
 	    n->pings_sent,
 	    n->pings_recv,
@@ -391,11 +402,12 @@ netdbSaveState(void *foo)
 	    (int) n->next_ping_time,
 	    (int) n->last_use_time);
 	for (x = n->hosts; x; x = x->next)
-	    fprintf(fp, " %s", x->name);
-	fprintf(fp, "\n");
+	    logfilePrintf(lf, " %s", hashKeyStr(&x->hash));
+	logfilePrintf(lf, "\n");
 	count++;
+#undef RBUF_SZ
     }
-    fclose(fp);
+    logfileClose(lf);
     getCurrentTime();
     debug(38, 1) ("NETDB state saved; %d entries, %d msec\n",
 	count, tvSubMsec(start, current_time));
@@ -408,75 +420,92 @@ netdbReloadState(void)
     LOCAL_ARRAY(char, path, SQUID_MAXPATHLEN);
     char *buf;
     char *t;
-    FILE *fp;
+    char *s;
+    int fd;
+    int l;
+    struct stat sb;
     netdbEntry *n;
     netdbEntry N;
     struct in_addr addr;
     int count = 0;
     struct timeval start = current_time;
     snprintf(path, SQUID_MAXPATHLEN, "%s/netdb_state", storeSwapDir(0));
-    fp = fopen(path, "r");
-    if (fp == NULL)
+    /*
+     * This was nicer when we were using stdio, but thanks to
+     * Solaris bugs, its a bad idea.  fopen can fail if more than
+     * 256 FDs are open.
+     */
+    fd = file_open(path, O_RDONLY | O_TEXT);
+    if (fd < 0)
 	return;
-    buf = memAllocate(MEM_4K_BUF);
-    while (fgets(buf, 4095, fp)) {
+    if (fstat(fd, &sb) < 0)
+	return;
+    t = buf = xcalloc(1, sb.st_size + 1);
+    l = read(fd, buf, sb.st_size);
+    file_close(fd);
+    if (l <= 0)
+	return;
+    while ((s = strchr(t, '\n'))) {
+	char *q;
+	assert(s - buf < l);
+	*s = '\0';
 	memset(&N, '\0', sizeof(netdbEntry));
-	if ((t = strtok(buf, w_space)) == NULL)
+	q = strtok(t, w_space);
+	t = s + 1;
+	if (NULL == q)
 	    continue;
-	if (!safe_inet_addr(t, &addr))
+	if (!safe_inet_addr(q, &addr))
 	    continue;
 	if (netdbLookupAddr(addr) != NULL)	/* no dups! */
 	    continue;
-	if ((t = strtok(NULL, w_space)) == NULL)
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.pings_sent = atoi(t);
-	if ((t = strtok(NULL, w_space)) == NULL)
+	N.pings_sent = atoi(q);
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.pings_recv = atoi(t);
+	N.pings_recv = atoi(q);
 	if (N.pings_recv == 0)
 	    continue;
 	/* give this measurement low weight */
 	N.pings_sent = 1;
 	N.pings_recv = 1;
-	if ((t = strtok(NULL, w_space)) == NULL)
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.hops = atof(t);
-	if ((t = strtok(NULL, w_space)) == NULL)
+	N.hops = atof(q);
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.rtt = atof(t);
-	if ((t = strtok(NULL, w_space)) == NULL)
+	N.rtt = atof(q);
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.next_ping_time = (time_t) atoi(t);
-	if ((t = strtok(NULL, w_space)) == NULL)
+	N.next_ping_time = (time_t) atoi(q);
+	if ((q = strtok(NULL, w_space)) == NULL)
 	    continue;
-	N.last_use_time = (time_t) atoi(t);
+	N.last_use_time = (time_t) atoi(q);
 	n = memAllocate(MEM_NETDBENTRY);
 	xmemcpy(n, &N, sizeof(netdbEntry));
 	netdbHashInsert(n, addr);
-	while ((t = strtok(NULL, w_space)) != NULL) {
-	    if (netdbLookupHost(t) != NULL)	/* no dups! */
+	while ((q = strtok(NULL, w_space)) != NULL) {
+	    if (netdbLookupHost(q) != NULL)	/* no dups! */
 		continue;
-	    netdbHostInsert(n, t);
+	    netdbHostInsert(n, q);
 	}
 	count++;
     }
-    memFree(buf, MEM_4K_BUF);
-    fclose(fp);
+    xfree(buf);
     getCurrentTime();
     debug(38, 1) ("NETDB state reloaded; %d entries, %d msec\n",
 	count, tvSubMsec(start, current_time));
 }
 
-static char *
+static const char *
 netdbPeerName(const char *name)
 {
-    wordlist *w;
+    const wordlist *w;
     for (w = peer_names; w; w = w->next) {
 	if (!strcmp(w->key, name))
 	    return w->key;
     }
-    w = wordlistAdd(&peer_names, name);
-    return w->key;
+    return wordlistAdd(&peer_names, name);
 }
 
 static void
@@ -491,7 +520,7 @@ static void
 netdbFreeNameEntry(void *data)
 {
     net_db_name *x = data;
-    xfree(x->name);
+    xfree(x->hash.key);
     memFree(x, MEM_NET_DB_NAME);
 }
 
@@ -569,6 +598,10 @@ netdbExchangeHandleReply(void *data, char *buf, ssize_t size)
 		o += sizeof(int);
 		hops = (double) ntohl(j) / 1000.0;
 		break;
+	    default:
+		debug(38, 1) ("netdbExchangeHandleReply: corrupt data, aborting\n");
+		netdbExchangeDone(ex);
+		return;
 	    }
 	}
 	if (addr.s_addr != any_addr.s_addr && rtt > 0)
@@ -592,11 +625,11 @@ netdbExchangeHandleReply(void *data, char *buf, ssize_t size)
 	netdbExchangeDone(ex);
     } else if (ex->e->store_status == STORE_PENDING) {
 	debug(38, 3) ("netdbExchangeHandleReply: STORE_PENDING\n");
-	storeClientCopy(ex->e, ex->seen, ex->used, ex->buf_sz,
+	storeClientCopy(ex->sc, ex->e, ex->seen, ex->used, ex->buf_sz,
 	    ex->buf, netdbExchangeHandleReply, ex);
     } else if (ex->seen < ex->e->mem_obj->inmem_hi) {
 	debug(38, 3) ("netdbExchangeHandleReply: ex->e->mem_obj->inmem_hi\n");
-	storeClientCopy(ex->e, ex->seen, ex->used, ex->buf_sz,
+	storeClientCopy(ex->sc, ex->e, ex->seen, ex->used, ex->buf_sz,
 	    ex->buf, netdbExchangeHandleReply, ex);
     } else {
 	debug(38, 3) ("netdbExchangeHandleReply: Done\n");
@@ -611,7 +644,7 @@ netdbExchangeDone(void *data)
     debug(38, 3) ("netdbExchangeDone: %s\n", storeUrl(ex->e));
     memFree(ex->buf, MEM_4K_BUF);
     requestUnlink(ex->r);
-    storeUnregister(ex->e, ex);
+    storeUnregister(ex->sc, ex->e, ex);
     storeUnlockObject(ex->e);
     cbdataUnlock(ex->p);
     cbdataFree(ex);
@@ -746,7 +779,7 @@ netdbDump(StoreEntry * sentry)
 	    n->rtt,
 	    n->hops);
 	for (x = n->hosts; x; x = x->next)
-	    storeAppendPrintf(sentry, " %s", x->name);
+	    storeAppendPrintf(sentry, " %s", hashKeyStr(&x->hash));
 	storeAppendPrintf(sentry, "\n");
 	p = n->peers;
 	for (j = 0; j < n->n_peers; j++, p++) {
@@ -886,6 +919,7 @@ void
 netdbBinaryExchange(StoreEntry * s)
 {
     http_reply *reply = s->mem_obj->reply;
+    http_version_t version;
 #if USE_ICMP
     netdbEntry *n;
     int i;
@@ -895,7 +929,8 @@ netdbBinaryExchange(StoreEntry * s)
     struct in_addr addr;
     storeBuffer(s);
     httpReplyReset(reply);
-    httpReplySetHeaders(reply, 1.0, HTTP_OK, "OK",
+    httpBuildVersion(&version, 1, 0);
+    httpReplySetHeaders(reply, version, HTTP_OK, "OK",
 	NULL, -1, squid_curtime, -2);
     httpReplySwapOut(reply, s);
     rec_sz = 0;
@@ -937,7 +972,8 @@ netdbBinaryExchange(StoreEntry * s)
     memFree(buf, MEM_4K_BUF);
 #else
     httpReplyReset(reply);
-    httpReplySetHeaders(reply, 1.0, HTTP_BAD_REQUEST, "Bad Request",
+    httpBuildVersion(&version, 1, 0);
+    httpReplySetHeaders(reply, version, HTTP_BAD_REQUEST, "Bad Request",
 	NULL, -1, squid_curtime, -2);
     storeAppendPrintf(s, "NETDB support not compiled into this Squid cache.\n");
 #endif
@@ -964,13 +1000,13 @@ netdbExchangeStart(void *data)
     }
     requestLink(ex->r);
     assert(NULL != ex->r);
-    ex->r->http_ver = 1.0;
+    httpBuildVersion(&ex->r->http_ver, 1, 0);
     ex->e = storeCreateEntry(uri, uri, null_request_flags, METHOD_GET);
-    ex->buf_sz = 4096;;
+    ex->buf_sz = 4096;
     ex->buf = memAllocate(MEM_4K_BUF);
     assert(NULL != ex->e);
-    storeClientListAdd(ex->e, ex);
-    storeClientCopy(ex->e, ex->seen, ex->used, ex->buf_sz,
+    ex->sc = storeClientListAdd(ex->e, ex);
+    storeClientCopy(ex->sc, ex->e, ex->seen, ex->used, ex->buf_sz,
 	ex->buf, netdbExchangeHandleReply, ex);
     ex->r->flags.loopdetect = 1;	/* cheat! -- force direct */
     if (p->login)
