@@ -1,21 +1,21 @@
 
 /*
- * $Id: peer_digest.c,v 1.70 1999/01/29 21:28:17 wessels Exp $
+ * $Id: peer_digest.c,v 1.75.2.1 2001/01/12 00:51:50 wessels Exp $
  *
  * DEBUG: section 72    Peer Digest Routines
  * AUTHOR: Alex Rousskov
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  Duane Wessels and the University of California San Diego.  Please
- *  see the COPYRIGHT file for full details.  Squid incorporates
- *  software developed and/or copyrighted by other sources.  Please see
- *  the CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -275,7 +275,12 @@ peerDigestRequest(PeerDigest * pd)
     pd->flags.requested = 1;
 
     /* compute future request components */
-    url = internalRemoteUri(p->host, p->http_port, "/squid-internal-periodic/", StoreDigestFileName);
+    if (p->digest_url)
+	url = xstrdup(p->digest_url);
+    else
+	url = internalRemoteUri(p->host, p->http_port,
+	    "/squid-internal-periodic/", StoreDigestFileName);
+
     key = storeKeyPublic(url, METHOD_GET);
     debug(72, 2) ("peerDigestRequest: %s key: %s\n", url, storeKeyText(key));
     req = urlParse(METHOD_GET, url);
@@ -285,7 +290,8 @@ peerDigestRequest(PeerDigest * pd)
     assert(!req->header.len);
     httpHeaderPutStr(&req->header, HDR_ACCEPT, StoreDigestMimeStr);
     httpHeaderPutStr(&req->header, HDR_ACCEPT, "text/html");
-
+    if (p->login)
+	xstrncpy(req->login, p->login, MAX_LOGIN_SZ);
     /* create fetch state structure */
     fetch = memAllocate(MEM_DIGEST_FETCH_STATE);
     cbdataAdd(fetch, memFree, MEM_DIGEST_FETCH_STATE);
@@ -306,21 +312,21 @@ peerDigestRequest(PeerDigest * pd)
 	debug(72, 5) ("peerDigestRequest: found old entry\n");
 	storeLockObject(old_e);
 	storeCreateMemObject(old_e, url, url);
-	storeClientListAdd(old_e, fetch);
+	fetch->old_sc = storeClientListAdd(old_e, fetch);
     }
     e = fetch->entry = storeCreateEntry(url, url, req->flags, req->method);
     assert(EBIT_TEST(e->flags, KEY_PRIVATE));
-    storeClientListAdd(e, fetch);
+    fetch->sc = storeClientListAdd(e, fetch);
     /* set lastmod to trigger IMS request if possible */
     if (old_e)
 	e->lastmod = old_e->lastmod;
 
     /* push towards peer cache */
     debug(72, 3) ("peerDigestRequest: forwarding to fwdStart...\n");
-    fwdStart(-1, e, req, no_addr, no_addr);
+    fwdStart(-1, e, req);
     cbdataLock(fetch);
     cbdataLock(fetch->pd);
-    storeClientCopy(e, 0, 0, 4096, memAllocate(MEM_4K_BUF),
+    storeClientCopy(fetch->sc, e, 0, 0, 4096, memAllocate(MEM_4K_BUF),
 	peerDigestFetchReply, fetch);
 }
 
@@ -330,17 +336,18 @@ peerDigestFetchReply(void *data, char *buf, ssize_t size)
 {
     DigestFetchState *fetch = data;
     PeerDigest *pd = fetch->pd;
+    size_t hdr_size;
     assert(pd && buf);
     assert(!fetch->offset);
 
     if (peerDigestFetchedEnough(fetch, buf, size, "peerDigestFetchReply"))
 	return;
 
-    if (headersEnd(buf, size)) {
+    if ((hdr_size = headersEnd(buf, size))) {
 	http_status status;
 	HttpReply *reply = fetch->entry->mem_obj->reply;
 	assert(reply);
-	httpReplyParse(reply, buf);
+	httpReplyParse(reply, buf, hdr_size);
 	status = reply->sline.status;
 	debug(72, 3) ("peerDigestFetchReply: %s status: %d, expires: %d (%+d)\n",
 	    strBuf(pd->host), status,
@@ -358,7 +365,7 @@ peerDigestFetchReply(void *data, char *buf, ssize_t size)
 	    httpReplyUpdateOnNotModified(fetch->old_entry->mem_obj->reply, reply);
 	    storeTimestampsSet(fetch->old_entry);
 	    /* get rid of 304 reply */
-	    storeUnregister(fetch->entry, fetch);
+	    storeUnregister(fetch->sc, fetch->entry, fetch);
 	    storeUnlockObject(fetch->entry);
 	    fetch->entry = fetch->old_entry;
 	    fetch->old_entry = NULL;
@@ -369,7 +376,7 @@ peerDigestFetchReply(void *data, char *buf, ssize_t size)
 	    /* get rid of old entry if any */
 	    if (fetch->old_entry) {
 		debug(72, 3) ("peerDigestFetchReply: got new digest, releasing old one\n");
-		storeUnregister(fetch->old_entry, fetch);
+		storeUnregister(fetch->old_sc, fetch->old_entry, fetch);
 		storeReleaseRequest(fetch->old_entry);
 		storeUnlockObject(fetch->old_entry);
 		fetch->old_entry = NULL;
@@ -384,14 +391,14 @@ peerDigestFetchReply(void *data, char *buf, ssize_t size)
 	if (status == HTTP_NOT_MODIFIED && fetch->pd->cd)
 	    peerDigestFetchStop(fetch, buf, "Not modified");
 	else
-	    storeClientCopy(fetch->entry,	/* have to swap in */
+	    storeClientCopy(fetch->sc, fetch->entry,	/* have to swap in */
 		0, 0, SM_PAGE_SIZE, buf, peerDigestSwapInHeaders, fetch);
     } else {
 	/* need more data, do we have space? */
 	if (size >= SM_PAGE_SIZE)
 	    peerDigestFetchAbort(fetch, buf, "reply header too big");
 	else
-	    storeClientCopy(fetch->entry, size, 0, SM_PAGE_SIZE, buf,
+	    storeClientCopy(fetch->sc, fetch->entry, size, 0, SM_PAGE_SIZE, buf,
 		peerDigestFetchReply, fetch);
     }
 }
@@ -410,7 +417,7 @@ peerDigestSwapInHeaders(void *data, char *buf, ssize_t size)
     if ((hdr_size = headersEnd(buf, size))) {
 	assert(fetch->entry->mem_obj->reply);
 	if (!fetch->entry->mem_obj->reply->sline.status)
-	    httpReplyParse(fetch->entry->mem_obj->reply, buf);
+	    httpReplyParse(fetch->entry->mem_obj->reply, buf, hdr_size);
 	if (fetch->entry->mem_obj->reply->sline.status != HTTP_OK) {
 	    debug(72, 1) ("peerDigestSwapInHeaders: %s status %d got cached!\n",
 		strBuf(fetch->pd->host), fetch->entry->mem_obj->reply->sline.status);
@@ -418,7 +425,7 @@ peerDigestSwapInHeaders(void *data, char *buf, ssize_t size)
 	    return;
 	}
 	fetch->offset += hdr_size;
-	storeClientCopy(fetch->entry, size, fetch->offset,
+	storeClientCopy(fetch->sc, fetch->entry, size, fetch->offset,
 	    SM_PAGE_SIZE, buf,
 	    peerDigestSwapInCBlock, fetch);
     } else {
@@ -426,7 +433,7 @@ peerDigestSwapInHeaders(void *data, char *buf, ssize_t size)
 	if (size >= SM_PAGE_SIZE)
 	    peerDigestFetchAbort(fetch, buf, "stored header too big");
 	else
-	    storeClientCopy(fetch->entry, size, 0, SM_PAGE_SIZE, buf,
+	    storeClientCopy(fetch->sc, fetch->entry, size, 0, SM_PAGE_SIZE, buf,
 		peerDigestSwapInHeaders, fetch);
     }
 }
@@ -452,7 +459,7 @@ peerDigestSwapInCBlock(void *data, char *buf, ssize_t size)
 	    memFree(buf, MEM_4K_BUF);
 	    buf = NULL;
 	    assert(pd->cd->mask);
-	    storeClientCopy(fetch->entry,
+	    storeClientCopy(fetch->sc, fetch->entry,
 		seen,
 		fetch->offset,
 		pd->cd->mask_size,
@@ -466,7 +473,7 @@ peerDigestSwapInCBlock(void *data, char *buf, ssize_t size)
 	if (size >= SM_PAGE_SIZE)
 	    peerDigestFetchAbort(fetch, buf, "digest cblock too big");
 	else
-	    storeClientCopy(fetch->entry, size, 0, SM_PAGE_SIZE, buf,
+	    storeClientCopy(fetch->sc, fetch->entry, size, 0, SM_PAGE_SIZE, buf,
 		peerDigestSwapInCBlock, fetch);
     }
 }
@@ -494,7 +501,7 @@ peerDigestSwapInMask(void *data, char *buf, ssize_t size)
     } else {
 	const size_t buf_sz = pd->cd->mask_size - fetch->mask_offset;
 	assert(buf_sz > 0);
-	storeClientCopy(fetch->entry,
+	storeClientCopy(fetch->sc, fetch->entry,
 	    fetch->offset,
 	    fetch->offset,
 	    buf_sz,
@@ -677,19 +684,19 @@ peerDigestFetchFinish(DigestFetchState * fetch, int err)
 
     if (fetch->old_entry) {
 	debug(72, 2) ("peerDigestFetchFinish: deleting old entry\n");
-	storeUnregister(fetch->old_entry, fetch);
+	storeUnregister(fetch->sc, fetch->old_entry, fetch);
 	storeReleaseRequest(fetch->old_entry);
 	storeUnlockObject(fetch->old_entry);
 	fetch->old_entry = NULL;
     }
     /* update global stats */
-    kb_incr(&Counter.cd.kbytes_sent, (size_t) fetch->sent.bytes);
-    kb_incr(&Counter.cd.kbytes_recv, (size_t) fetch->recv.bytes);
-    Counter.cd.msgs_sent += fetch->sent.msg;
-    Counter.cd.msgs_recv += fetch->recv.msg;
+    kb_incr(&statCounter.cd.kbytes_sent, (size_t) fetch->sent.bytes);
+    kb_incr(&statCounter.cd.kbytes_recv, (size_t) fetch->recv.bytes);
+    statCounter.cd.msgs_sent += fetch->sent.msg;
+    statCounter.cd.msgs_recv += fetch->recv.msg;
 
     /* unlock everything */
-    storeUnregister(fetch->entry, fetch);
+    storeUnregister(fetch->sc, fetch->entry, fetch);
     storeUnlockObject(fetch->entry);
     requestUnlink(fetch->request);
     fetch->entry = NULL;
@@ -792,7 +799,7 @@ peerDigestSetCBlock(PeerDigest * pd, const char *buf)
 	    host, cblock.mask_size, (int) (cblock.mask_size - freed_size));
 	pd->cd = cacheDigestCreate(cblock.capacity, cblock.bits_per_entry);
 	if (cblock.mask_size >= freed_size)
-	    kb_incr(&Counter.cd.memory, cblock.mask_size - freed_size);
+	    kb_incr(&statCounter.cd.memory, cblock.mask_size - freed_size);
     }
     assert(pd->cd);
     /* these assignments leave us in an inconsistent state until we finish reading the digest */
