@@ -1,21 +1,21 @@
 
 /*
- * $Id: stat.c,v 1.319.2.6 2000/02/09 23:30:01 wessels Exp $
+ * $Id: stat.c,v 1.340.2.2 2001/05/27 23:52:21 hno Exp $
  *
  * DEBUG: section 18    Cache Manager Statistics
  * AUTHOR: Harvest Derived
  *
- * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
  * ----------------------------------------------------------
  *
- *  Squid is the result of efforts by numerous individuals from the
- *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by the
- *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
- *  the Regents of the University of California.  Please see the
- *  COPYRIGHT file for full details.  Squid incorporates software
- *  developed and/or copyrighted by other sources.  Please see the
- *  CREDITS file for full details.
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -79,7 +79,9 @@ static OBJH statCountersHistograms;
 static OBJH statClientRequests;
 
 #ifdef XMALLOC_STATISTICS
-static void info_get_mallstat(int, int, void *);
+static void info_get_mallstat(int, int, int, void *);
+static double xm_time;
+static double xm_deltat;
 #endif
 
 StatCounters CountHist[N_COUNT_HIST];
@@ -87,6 +89,9 @@ static int NCountHist = 0;
 static StatCounters CountHourHist[N_COUNT_HOUR_HIST];
 static int NCountHourHist = 0;
 
+extern unsigned int mem_pool_alloc_calls;
+extern unsigned int mem_pool_free_calls;
+
 static void
 statUtilization(StoreEntry * e)
 {
@@ -253,7 +258,8 @@ statStoreEntry(StoreEntry * s, StoreEntry * e)
     MemObject *mem = e->mem_obj;
     int i;
     struct _store_client *sc;
-    storeAppendPrintf(s, "KEY %s\n", storeKeyText(e->key));
+    dlink_node *node;
+    storeAppendPrintf(s, "KEY %s\n", storeKeyText(e->hash.key));
     if (mem)
 	storeAppendPrintf(s, "\t%s %s\n",
 	    RequestMethodStr[mem->method], mem->log_url);
@@ -264,8 +270,8 @@ statStoreEntry(StoreEntry * s, StoreEntry * e)
 	(int) e->lock_count,
 	storePendingNClients(e),
 	(int) e->refcount);
-    storeAppendPrintf(s, "\tSwap File %#08X\n",
-	e->swap_file_number);
+    storeAppendPrintf(s, "\tSwap Dir %d, File %#08X\n",
+	e->swap_dirn, e->swap_filen);
     if (mem != NULL) {
 	storeAppendPrintf(s, "\tinmem_lo: %d\n", (int) mem->inmem_lo);
 	storeAppendPrintf(s, "\tinmem_hi: %d\n", (int) mem->inmem_hi);
@@ -274,7 +280,8 @@ statStoreEntry(StoreEntry * s, StoreEntry * e)
 	if (mem->swapout.sio)
 	    storeAppendPrintf(s, "\tswapout: %d bytes written\n",
 		(int) storeOffset(mem->swapout.sio));
-	for (i = 0, sc = &mem->clients[i]; sc != NULL; sc = sc->next, i++) {
+	for (i = 0, node = mem->clients.head; node; node = node->next, i++) {
+	    sc = (store_client *) node->data;
 	    if (sc->callback_data == NULL)
 		continue;
 	    storeAppendPrintf(s, "\tClient #%d, %p\n", i, sc->callback_data);
@@ -383,11 +390,11 @@ statOpenfdObj(StoreEntry * sentry)
 
 #ifdef XMALLOC_STATISTICS
 static void
-info_get_mallstat(int size, int number, void *data)
+info_get_mallstat(int size, int number, int oldnum, void *data)
 {
     StoreEntry *sentry = data;
     if (number > 0)
-	storeAppendPrintf(sentry, "\t%d = %d\n", size, number);
+	storeAppendPrintf(sentry, "%d\t %d\t %d\t %.1f\n", size, number, number - oldnum, xdiv((number - oldnum), xm_deltat));
 }
 #endif
 
@@ -458,24 +465,24 @@ info_get(StoreEntry * sentry)
     storeAppendPrintf(sentry, "Connection information for %s:\n",
 	appname);
     storeAppendPrintf(sentry, "\tNumber of clients accessing cache:\t%u\n",
-	Counter.client_http.clients);
+	statCounter.client_http.clients);
     storeAppendPrintf(sentry, "\tNumber of HTTP requests received:\t%u\n",
-	Counter.client_http.requests);
+	statCounter.client_http.requests);
     storeAppendPrintf(sentry, "\tNumber of ICP messages received:\t%u\n",
-	Counter.icp.pkts_recv);
+	statCounter.icp.pkts_recv);
     storeAppendPrintf(sentry, "\tNumber of ICP messages sent:\t%u\n",
-	Counter.icp.pkts_sent);
+	statCounter.icp.pkts_sent);
     storeAppendPrintf(sentry, "\tNumber of queued ICP replies:\t%u\n",
-	Counter.icp.replies_queued);
+	statCounter.icp.replies_queued);
     storeAppendPrintf(sentry, "\tRequest failure ratio:\t%5.2f%%\n",
 	request_failure_ratio);
 
     storeAppendPrintf(sentry, "\tHTTP requests per minute:\t%.1f\n",
-	Counter.client_http.requests / (runtime / 60.0));
+	statCounter.client_http.requests / (runtime / 60.0));
     storeAppendPrintf(sentry, "\tICP messages per minute:\t%.1f\n",
-	(Counter.icp.pkts_sent + Counter.icp.pkts_recv) / (runtime / 60.0));
+	(statCounter.icp.pkts_sent + statCounter.icp.pkts_recv) / (runtime / 60.0));
     storeAppendPrintf(sentry, "\tSelect loop called: %d times, %0.3f ms avg\n",
-	Counter.select_loops, 1000.0 * runtime / Counter.select_loops);
+	statCounter.select_loops, 1000.0 * runtime / statCounter.select_loops);
 
     storeAppendPrintf(sentry, "Cache information for %s:\n",
 	appname);
@@ -485,21 +492,20 @@ info_get(StoreEntry * sentry)
     storeAppendPrintf(sentry, "\tByte Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
 	statByteHitRatio(5),
 	statByteHitRatio(60));
+    storeAppendPrintf(sentry, "\tRequest Memory Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
+	statRequestHitMemoryRatio(5),
+	statRequestHitMemoryRatio(60));
+    storeAppendPrintf(sentry, "\tRequest Disk Hit Ratios:\t5min: %3.1f%%, 60min: %3.1f%%\n",
+	statRequestHitDiskRatio(5),
+	statRequestHitDiskRatio(60));
     storeAppendPrintf(sentry, "\tStorage Swap size:\t%d KB\n",
 	store_swap_size);
     storeAppendPrintf(sentry, "\tStorage Mem size:\t%d KB\n",
 	(int) (store_mem_size >> 10));
-#if HEAP_REPLACEMENT
-    storeAppendPrintf(sentry, "\tStorage Replacement Threshold:\t%f\n",
-	heap_peepminkey(store_heap));
-#else
-    storeAppendPrintf(sentry, "\tStorage LRU Expiration Age:\t%6.2f days\n",
-	(double) storeExpiredReferenceAge() / 86400.0);
-#endif
     storeAppendPrintf(sentry, "\tMean Object Size:\t%0.2f KB\n",
 	n_disk_objects ? (double) store_swap_size / n_disk_objects : 0.0);
     storeAppendPrintf(sentry, "\tRequests given to unlinkd:\t%d\n",
-	Counter.unlink.requests);
+	statCounter.unlink.requests);
 
     storeAppendPrintf(sentry, "Median Service Times (seconds)  5 min    60 min:\n");
     storeAppendPrintf(sentry, "\tHTTP Requests (All):  %8.5f %8.5f\n",
@@ -586,6 +592,10 @@ info_get(StoreEntry * sentry)
     storeAppendPrintf(sentry, "Memory accounted for:\n");
     storeAppendPrintf(sentry, "\tTotal accounted:       %6d KB\n",
 	memTotalAllocated() >> 10);
+    storeAppendPrintf(sentry, "\tmemPoolAlloc calls: %d\n",
+	mem_pool_alloc_calls);
+    storeAppendPrintf(sentry, "\tmemPoolFree calls: %d\n",
+	mem_pool_free_calls);
 
     storeAppendPrintf(sentry, "File descriptor usage for %s:\n", appname);
     storeAppendPrintf(sentry, "\tMaximum number of file descriptors:   %4d\n",
@@ -610,13 +620,14 @@ info_get(StoreEntry * sentry)
 	memInUse(MEM_MEMOBJECT));
     storeAppendPrintf(sentry, "\t%6d Hot Object Cache Items\n",
 	hot_obj_count);
-    storeAppendPrintf(sentry, "\t%6d Filemap bits set\n",
-	storeDirMapBitsInUse());
     storeAppendPrintf(sentry, "\t%6d on-disk objects\n",
 	n_disk_objects);
 
 #if XMALLOC_STATISTICS
-    storeAppendPrintf(sentry, "Memory allocation statistics\n");
+    xm_deltat = current_dtime - xm_time;
+    xm_time = current_dtime;
+    storeAppendPrintf(sentry, "\nMemory allocation statistics\n");
+    storeAppendPrintf(sentry, "Allocation Size\t Alloc Count\t Alloc Delta\t Allocs/sec \n");
     malloc_statistics(info_get_mallstat, sentry);
 #endif
 }
@@ -780,8 +791,12 @@ statAvgDump(StoreEntry * sentry, int minutes, int hours)
 	: 0.0);
     x = statHistDeltaMedian(&l->select_fds_hist, &f->select_fds_hist);
     storeAppendPrintf(sentry, "median_select_fds = %f\n", x);
-    storeAppendPrintf(sentry, "swap_files_cleaned = %f/sec\n",
-	XAVG(swap_files_cleaned));
+    storeAppendPrintf(sentry, "swap.outs = %f/sec\n",
+	XAVG(swap.outs));
+    storeAppendPrintf(sentry, "swap.ins = %f/sec\n",
+	XAVG(swap.ins));
+    storeAppendPrintf(sentry, "swap.files_cleaned = %f/sec\n",
+	XAVG(swap.files_cleaned));
     storeAppendPrintf(sentry, "aborted_requests = %f/sec\n",
 	XAVG(aborted_requests));
 
@@ -821,7 +836,7 @@ statInit(void)
 	statCountersInit(&CountHist[i]);
     for (i = 0; i < N_COUNT_HOUR_HIST; i++)
 	statCountersInit(&CountHourHist[i]);
-    statCountersInit(&Counter);
+    statCountersInit(&statCounter);
     eventAdd("statAvgTick", statAvgTick, NULL, (double) COUNT_INTERVAL, 1);
     cachemgrRegister("info",
 	"General Runtime Information",
@@ -881,7 +896,7 @@ statAvgTick(void *notused)
 {
     StatCounters *t = &CountHist[0];
     StatCounters *p = &CountHist[1];
-    StatCounters *c = &Counter;
+    StatCounters *c = &statCounter;
     struct rusage rusage;
     eventAdd("statAvgTick", statAvgTick, NULL, (double) COUNT_INTERVAL, 1);
     squid_getrusage(&rusage);
@@ -904,6 +919,32 @@ statAvgTick(void *notused)
 	statCountersCopy(t, c);
 	NCountHourHist++;
     }
+    if (Config.warnings.high_rptm > 0) {
+	int i = (int) statMedianSvc(20, MEDIAN_HTTP);
+	if (Config.warnings.high_rptm < i)
+	    debug(18, 0) ("WARNING: Median response time is %d milliseconds\n", i);
+    }
+    if (Config.warnings.high_pf) {
+	int i = (CountHist[0].page_faults - CountHist[1].page_faults);
+	double dt = tvSubDsec(CountHist[0].timestamp, CountHist[1].timestamp);
+	if (i > 0 && dt > 0.0) {
+	    i /= (int) dt;
+	    if (Config.warnings.high_pf < i)
+		debug(18, 0) ("WARNING: Page faults occuring at %d/sec\n", i);
+	}
+    }
+    if (Config.warnings.high_memory) {
+	int i = 0;
+#if HAVE_MSTATS && HAVE_GNUMALLOC_H
+	struct mstats ms = mstats();
+	i = ms.bytes_total;
+#elif HAVE_MALLINFO && HAVE_STRUCT_MALLINFO
+	struct mallinfo mp = mallinfo();
+	i = mp.arena;
+#endif
+	if (Config.warnings.high_memory < i)
+	    debug(18, 0) ("WARNING: Memory usage at %d MB\n", i >> 20);
+    }
 }
 
 static void
@@ -961,6 +1002,7 @@ statCountersClean(StatCounters * C)
     statHistClean(&C->dns.svc_time);
     statHistClean(&C->cd.on_xition_count);
     statHistClean(&C->comm_icp_incoming);
+    statHistClean(&C->comm_dns_incoming);
     statHistClean(&C->comm_http_incoming);
     statHistClean(&C->select_fds_hist);
 }
@@ -971,7 +1013,7 @@ statCountersCopy(StatCounters * dest, const StatCounters * orig)
 {
     assert(dest && orig);
     /* this should take care of all the fields, but "special" ones */
-    memcpy(dest, orig, sizeof(*dest));
+    xmemcpy(dest, orig, sizeof(*dest));
     /* prepare space where to copy special entries */
     statCountersInitSpecial(dest);
     /* now handle special cases */
@@ -993,7 +1035,7 @@ statCountersCopy(StatCounters * dest, const StatCounters * orig)
 static void
 statCountersHistograms(StoreEntry * sentry)
 {
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     storeAppendPrintf(sentry, "client_http.all_svc_time histogram:\n");
     statHistDump(&f->client_http.all_svc_time, sentry, NULL);
     storeAppendPrintf(sentry, "client_http.miss_svc_time histogram:\n");
@@ -1015,7 +1057,7 @@ statCountersHistograms(StoreEntry * sentry)
 static void
 statCountersDump(StoreEntry * sentry)
 {
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     struct rusage rusage;
     squid_getrusage(&rusage);
     f->page_faults = rusage_pagefaults(&rusage);
@@ -1132,8 +1174,12 @@ statCountersDump(StoreEntry * sentry)
 	f->cputime);
     storeAppendPrintf(sentry, "wall_time = %f\n",
 	tvSubDsec(f->timestamp, current_time));
-    storeAppendPrintf(sentry, "swap_files_cleaned = %d\n",
-	f->swap_files_cleaned);
+    storeAppendPrintf(sentry, "swap.outs = %d\n",
+	f->swap.outs);
+    storeAppendPrintf(sentry, "swap.ins = %d\n",
+	f->swap.ins);
+    storeAppendPrintf(sentry, "swap.files_cleaned = %d\n",
+	f->swap.files_cleaned);
     storeAppendPrintf(sentry, "aborted_requests = %d\n",
 	f->aborted_requests);
 }
@@ -1152,7 +1198,7 @@ static void
 statPeerSelect(StoreEntry * sentry)
 {
 #if USE_CACHE_DIGESTS
-    StatCounters *f = &Counter;
+    StatCounters *f = &statCounter;
     peer *peer;
     const int tot_used = f->cd.times_used + f->icp.times_used;
 
@@ -1268,7 +1314,7 @@ int
 stat5minClientRequests(void)
 {
     assert(N_COUNT_HIST > 5);
-    return Counter.client_http.requests - CountHist[5].client_http.requests;
+    return statCounter.client_http.requests - CountHist[5].client_http.requests;
 }
 
 static double
@@ -1289,15 +1335,51 @@ statRequestHitRatio(int minutes)
 	CountHist[minutes].client_http.requests);
 }
 
+extern double
+statRequestHitMemoryRatio(int minutes)
+{
+    assert(minutes < N_COUNT_HIST);
+    return dpercent(CountHist[0].client_http.mem_hits -
+	CountHist[minutes].client_http.mem_hits,
+	CountHist[0].client_http.hits -
+	CountHist[minutes].client_http.hits);
+}
+
+extern double
+statRequestHitDiskRatio(int minutes)
+{
+    assert(minutes < N_COUNT_HIST);
+    return dpercent(CountHist[0].client_http.disk_hits -
+	CountHist[minutes].client_http.disk_hits,
+	CountHist[0].client_http.hits -
+	CountHist[minutes].client_http.hits);
+}
+
 extern double
 statByteHitRatio(int minutes)
 {
     size_t s;
     size_t c;
+#if USE_CACHE_DIGESTS
+    size_t cd;
+#endif
+    /* size_t might be unsigned */
     assert(minutes < N_COUNT_HIST);
     c = CountHist[0].client_http.kbytes_out.kb - CountHist[minutes].client_http.kbytes_out.kb;
     s = CountHist[0].server.all.kbytes_in.kb - CountHist[minutes].server.all.kbytes_in.kb;
-    /* size_t might be unsigned */
+#if USE_CACHE_DIGESTS
+    /*
+     * This ugly hack is here to prevent the user from seeing a
+     * negative byte hit ratio.  When we fetch a cache digest from
+     * a neighbor, it gets treated like a cache miss because the
+     * object is consumed internally.  Thus, we subtract cache
+     * digest bytes out before calculating the byte hit ratio.
+     */
+    cd = CountHist[0].cd.kbytes_recv.kb - CountHist[minutes].cd.kbytes_recv.kb;
+    if (s < cd)
+	debug(18, 1) ("STRANGE: srv_kbytes=%d, cd_kbytes=%d\n", s, cd);
+    s -= cd;
+#endif
     if (c > s)
 	return dpercent(c - s, c);
     else
@@ -1343,9 +1425,9 @@ statClientRequests(StoreEntry * s)
 	    http->out.offset, http->out.size);
 	storeAppendPrintf(s, "req_sz %d\n", http->req_sz);
 	e = http->entry;
-	storeAppendPrintf(s, "entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
+	storeAppendPrintf(s, "entry %p/%s\n", e, e ? storeKeyText(e->hash.key) : "N/A");
 	e = http->old_entry;
-	storeAppendPrintf(s, "old_entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
+	storeAppendPrintf(s, "old_entry %p/%s\n", e, e ? storeKeyText(e->hash.key) : "N/A");
 	storeAppendPrintf(s, "start %d.%06d (%f seconds ago)\n", http->start.tv_sec,
 	    http->start.tv_usec,
 	    tvSubDsec(http->start, current_time));
